import argparse
import subprocess
import concurrent.futures
import threading


parser = argparse.ArgumentParser(description="run")
parser.add_argument("--device", type=str)
parser.add_argument("--run", type=int)

args = parser.parse_args()
max_workers = 4

class GPUManager:
    def __init__(self, num_gpus, max_jobs_per_gpu):
        self.num_gpus = num_gpus
        self.max_jobs_per_gpu = max_jobs_per_gpu
        self.gpu_lock = threading.Lock()
        self.jobs_count = {gpu_idx: 0 for gpu_idx in range(num_gpus)}

    def get_gpu(self):
        with self.gpu_lock:
            available_gpus = [gpu_idx for gpu_idx, jobs_count in self.jobs_count.items() if
                              jobs_count < self.max_jobs_per_gpu]
            if not available_gpus:
                return None  # No available GPU with space for more jobs
            gpu_idx = min(available_gpus, key=lambda x: self.jobs_count[x])
            self.jobs_count[gpu_idx] += 1
            # print(f"GPU {gpu_idx} has {self.jobs_count[gpu_idx]} jobs running")

        return gpu_idx

    def free_gpu(self, gpu_idx):
        with self.gpu_lock:
            if gpu_idx in self.jobs_count and self.jobs_count[gpu_idx] > 0:
                self.jobs_count[gpu_idx] -= 1


def run_exp_wgpu(cmd_line, gpu_manager):
    gpu_idx = gpu_manager.get_gpu() + 2
    print(f"Running on GPU {gpu_idx}")
    # print(f"Running on GPU {gpu_idx}")

    cmd_line = cmd_line.format(gpu_idx)
    subprocess.call(cmd_line, shell=True)
    gpu_manager.free_gpu(gpu_idx)

if args.run == 0:
    edge_type = "multi_relational_degree_lj_pot"
    dataset = "QM9"
    mode_list = ["random"]

    mode = "random"
    split = "valid"

    layers = [8]
    learning_rate = [9e-5]

    num_gpus = 8
    max_jobs_per_gpu = 1
    num_workers = max_jobs_per_gpu * num_gpus

    gpu_manager = GPUManager(num_gpus=num_gpus, max_jobs_per_gpu=max_jobs_per_gpu)
    futures = []
    with concurrent.futures.ThreadPoolExecutor(max_workers=num_workers) as executor:
        for mode in mode_list:
            for ly in layers:
                for lr in learning_rate:
                    print(f"Running {ly} layers with lr {lr}")

                    # run training
                    cmd_line = f"bash experiments/conformer_prediction/ours_for_conformer_prediction.sh {{}} {dataset} {mode} {ly} ours-{dataset}-{mode}-ljpot-final-lr{lr} {edge_type} -1 {lr}"
                    future = executor.submit(run_exp_wgpu, cmd_line, gpu_manager)
                    futures.append(future)
        concurrent.futures.wait(futures)
