from datasets import load_dataset, DatasetDict, concatenate_datasets


def custom_split(dist_type, train, eval, test):
    if dist_type == 0:
        # default split
        return train, eval, test, None
    elif dist_type == 1:
        # Step 1: Concatenate the datasets
        total_set = concatenate_datasets([train, eval, test])

        # Step 2: Calculate molecule sizes (length of 'node_attr' for each element)
        molecule_sizes = [len(attr) for attr in total_set['node_attr']]

        # Step 3: Sort the dataset by molecule size
        sorted_indices = sorted(range(len(molecule_sizes)), key=lambda i: molecule_sizes[i])
        test_indices = sorted_indices[len(test)-len(sorted_indices):]
        sorted_total_set = total_set.select(sorted_indices)

        # Step 4: Determine the original sizes of the splits
        train_size = len(train)
        eval_size = len(eval)
        test_size = len(test)

        # Step 5: Split the sorted dataset into train, eval, and test sets
        sorted_train_set = sorted_total_set.select(range(train_size))
        sorted_eval_set = sorted_total_set.select(range(train_size, train_size + eval_size))
        sorted_test_set = sorted_total_set.select(range(train_size + eval_size, train_size + eval_size + test_size))

        # Now you have the sorted and split datasets
        train_set = sorted_train_set.shuffle()
        eval_set = sorted_eval_set.shuffle()
        test_set = sorted_test_set.shuffle()

    else:
        raise ValueError("Invalid distribution type.")

    return train_set, eval_set, test_set, test_indices

