import torch

def merge_edges_and_attributes(original_edges, original_attr, new_edges, new_attr):
    # Concatenate edges and attributes
    all_edges = torch.cat([original_edges, new_edges], dim=1).long()
    # edge attributes are embedded, and thus require to be of longtype
    if original_attr is None:
        all_attr = None
    else:
        all_attr = torch.cat([original_attr, new_attr], dim=0).long()

    return all_edges, all_attr

def generate_gt_augmentation(conformer, edge_index, edge_attr):
    diff = conformer.unsqueeze(1) - conformer.unsqueeze(0)
    dist_matrix = torch.sqrt((diff ** 2).sum(-1))

    # cutoff distance of 5.0 Angstroms, since most forces work within this cutoff
    cutoff_distance = 5.0

    mask = dist_matrix < cutoff_distance
    new_edges = mask.nonzero(as_tuple=False).t()
    new_edge_attr = torch.stack([torch.ones(4) for _ in range(new_edges.shape[1])])

    edge_index, edge_attr = merge_edges_and_attributes(
        edge_index,
        edge_attr,
        new_edges,
        new_edge_attr
    )

    return edge_index, edge_attr

